//---------------------------------------------------------------------------
// BBMobile_Demo_App
//---------------------------------------------------------------------------
// This is demo sketch
// you can freely change and redistribute it - dont forget to add link to our website:
// www.bbmagic.net
// To jest demonstracyjny sketch.
// możesz go dowolnie modyfikować i redystrybuować - nie zapomnij tylko o zawarciu linku:
// www.bbmagic.net
//---------------------------------------------------------------------------
#include <avr/pgmspace.h>
#include <SoftwareSerial.h>
#include "bbmobile_arduino_01.h"

// JSON definition of mobile interface - it is size optimized
const char AppJson[] PROGMEM ="{\"ty\":\"lout\",\"or\":\"V\",\"img\":\"1\",\"cs\":[ \
{\"ty\":\"TextView\",\"w\":\"1\"},  \
{\"ty\":\"TextView\",\"n\":\"t1\",\"te\":\"ARDUINO\nDEMO APP\",\"tc\":\"255,255,255\",\"ts\":\"50\",\"w\":\"2\",\"tf\":\"03\"}, \
{\"ty\":\"TextView\",\"w\":\"2\"},  \
{\"ty\":\"Button\",\"n\":\"b1\",\"te\":\"Like it\",\"tc\":\"255,255,255\",\"ts\":\"40\",\"bg\":\"109,83,146\",\"w\":\"2\"}, \
{\"ty\":\"TextView\",\"w\":\"1\"},  \
{\"ty\":\"Button\",\"n\":\"b2\",\"te\":\"Love it\",\"tc\":\"255,255,255\",\"ts\":\"40\",\"bg\":\"255,75,75\",\"w\":\"2\"},  \
{\"ty\":\"TextView\",\"w\":\"4\"}]}" ;

// software serial port for communication with BBMobile module
SoftwareSerial bbmSerial(2, 3); // RX, TX

//PINS for BBMobile powering
#define BBMOBILE_GND_PIN       4
#define BBMOBILE_POWER_PIN     5


//--------------------------------------------------
// setup code here, to run once
//--------------------------------------------------
void setup()
{
  // initialize digital pin LED_BUILTIN as an output
  pinMode(LED_BUILTIN, OUTPUT);
  
  // for powering BBMobile from defined pins
  pinMode(BBMOBILE_GND_PIN, OUTPUT) ;
  digitalWrite(BBMOBILE_GND_PIN, LOW);    //-GND for BBMobile module
  pinMode(BBMOBILE_POWER_PIN, OUTPUT) ;
  digitalWrite(BBMOBILE_POWER_PIN, HIGH); //-VCC for BBMobile module
  
  // set the data rate for the Hardware Serial port
  Serial.begin(9600) ;
  while(!Serial) ;

  Serial.println("\n\nSTART BBMOBILE DEMO APP") ;  

  // set the data rate for the BBMobile Software Serial port and set size of bbm_buf
  bbmSerial.begin(9600) ;
  bbm_buf.reserve(_SS_MAX_RX_BUFF) ;
  
  //-check serial communication with BBMobile module
  //------------------------------------------------
  Serial.print("Searching for BBMobile") ;  
  do {
    Serial.print(".") ;
    digitalWrite(LED_BUILTIN, HIGH);   //-turn the LED on
    delay(500) ;    
    digitalWrite(LED_BUILTIN, LOW);    //-turn the LED off
    delay(500);         
  }while(BBMobileSend(&bbmSerial, "<hello")) ;
  Serial.println("FOUND") ;  

  //-set BBMobile modules name
  //----------------------------------  
  Serial.print("Setting mobile name - ") ;  
  if(BBMobileSend(&bbmSerial, "<name,ARDUINO DEMO APP"))
  {
    Serial.println("err") ;
  }else Serial.println("OK") ;  

  //-set BBMobile modules PIN
  //----------------------------------    
  //Serial.print("Setting PIN - ") ;    
  //if( BBMobileSend(&bbmSerial, "<pin,123") )  //-set PIN to 123  
  Serial.print("Setting no PIN - ") ;    
  if( BBMobileSend(&bbmSerial, "<pin,0") )  //-delete PIN
  {
    Serial.println("err") ;
  }else Serial.println("OK") ;  
}

//--------------------------------------------------
// main code here, to run repeatedly
//--------------------------------------------------
void loop()
{

  //-wait for Bluetooth LE connection
  //----------------------------------    
  Serial.print("Waiting for App connection...") ;  
  do {
    digitalWrite(LED_BUILTIN, LOW);    // turn the LED off by making the voltage LOW
    delay(900) ;    
    digitalWrite(LED_BUILTIN, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(100) ;             
    BBMobileGetMessage(&bbmSerial) ;
  }while(BBMobileIsConnected() == 0) ;
  Serial.println("OK") ;  

  //-send JSON to build mobile interface 
  //------------------------------------
  Serial.print("Sending JSON...") ;  
  if(BBMobileSendJson(&bbmSerial, AppJson))
  {
    Serial.println("err") ;
  }else Serial.println("OK\nPlaying with interface...") ;  
  
  //-play with interface
  //------------------------------------
  while(BBMobileIsConnected())
  {
    delay(100) ;
    
    //-get messages from mobile interface
    //------------------------------------
    while(BBMobileGetMessage(&bbmSerial) > 0)
    {
      //Serial.println("msg: " + bbm_buf) ;
      if(bbm_buf.substring(4, 6) == "b1")  //- after b1 button tap - in bbm_buf: $by,b1"1"
      {
        Serial.println("BUTTON Like it") ;  
        
        //-change only text
        //BBMobileSend(&bbmSerial, "$set,t1:te=\"I like it too !!\"") ;
        
        //-or change text and color        
        //BBMobileSend(&bbmSerial, "$set,t1:tc=\"109,83,146\"te=\"I like it too !!\"") ;
        
        //-or blink
        for(int i=0; i<3; i++)
        {
          BBMobileSend(&bbmSerial, "$set,t1:tc=\"109,83,146\"te=\"I like it too !!\"") ;  
          delay(75) ;
          BBMobileSend(&bbmSerial, "$set,t1:te=\" \\n \"") ;  
          delay(75) ;
        }
        BBMobileSend(&bbmSerial, "$set,t1:tc=\"255,255,255\"te=\"ARDUINO\\nDEMO APP\"") ;

      }else if(bbm_buf.substring(4, 6) == "b2")  //- after b1 button tap - in bbm_buf: $by,b2"1"
      {
        Serial.println("BUTTON Love it") ;  
        
        //-change only text
        //BBMobileSend(&bbmSerial, "$set,t1:te=\"Thank\\nYou\"") ;        
        
        //-or change text and color                
        BBMobileSend(&bbmSerial, "$set,t1:tc=\"255,75,75\"te=\"Thank\\nYou !!\"") ;                
      }      
    } ;
    
    //-send data to mobile interface
    //------------------------------------    
    // no data to send in this demo sketch
	    
  } ;    

  //-Here Bluetooth LE is disconnected
  //----------------------------------    
  Serial.println("App disconnected") ;      
}
