//---------------------------------------------------------------------------
// BBMobile_Volt_Meter
//---------------------------------------------------------------------------
// This is demo sketch
// you can freely change and redistribute it - dont forget to add link to our website:
// www.bbmagic.net
// To jest demonstracyjny sketch.
// możesz go dowolnie modyfikować i redystrybuować - nie zapomnij tylko o zawarciu linku:
// www.bbmagic.net
//---------------------------------------------------------------------------

#include <avr/pgmspace.h>
#include <SoftwareSerial.h>
#include "bbmobile_arduino_01.h"

// JSON definition of mobile interface - it is size optimized
const char AppJson[] PROGMEM ="{\"ty\":\"lout\",\"or\":\"V\",\"img\":\"3\",\"cs\":[  \
{\"ty\":\"TextView\",\"w\":\"1\"},  \
{\"ty\":\"TextView\",\"n\":\"t1\",\"tc\":\"255,255,255\",\"ts\":\"40\",\"w\":\"2\",\"tf\":\"02\"},  \
{\"ty\":\"TextView\",\"n\":\"t2\",\"tc\":\"255,255,255\",\"ts\":\"40\",\"w\":\"2\",\"tf\":\"02\"},  \
{\"ty\":\"TextView\",\"n\":\"t3\",\"tc\":\"255,255,255\",\"ts\":\"40\",\"w\":\"2\",\"tf\":\"02\"},  \
{\"ty\":\"TextView\",\"w\":\"5\"}]}" ;

// software serial port for communication with BBMobile BLE module
SoftwareSerial bbmSerial(2, 3); // RX, TX

//PINS for BBMobile powering
#define BBMOBILE_GND_PIN       4
#define BBMOBILE_POWER_PIN     5

// uncomment this below if you want to see demo version without real measurements
//#define DEMO

//--------------------------------------------------
// setup code here, to run once
//--------------------------------------------------
void setup()
{
  // initialize digital pin LED_BUILTIN as an output
  pinMode(LED_BUILTIN, OUTPUT);

  // for powering BBMobile from defined pins
  pinMode(BBMOBILE_GND_PIN, OUTPUT) ;
  digitalWrite(BBMOBILE_GND_PIN, LOW);    //-GND for BBMobile module
  pinMode(BBMOBILE_POWER_PIN, OUTPUT) ;
  digitalWrite(BBMOBILE_POWER_PIN, HIGH); //-VCC for BBMobile module
  
  // set the data rate for the Hardware Serial port
  Serial.begin(9600) ;
  while(!Serial) ;

  
  #ifdef DEMO
    Serial.println("\n\nSTART MOBILE V METER - DEMO VERSION") ;    
  #else  
    Serial.println("\n\nSTART MOBILE V METER") ;    
  #endif  //-def DEMO        

  // set the data rate for the BBMobile SoftwareSerial port and set size of bbm_buf
  bbmSerial.begin(9600) ;
  bbm_buf.reserve(_SS_MAX_RX_BUFF) ;
  
  //-check serial communication with BBMobile module
  //------------------------------------------------
  Serial.print("Searching for BBMobile") ;  
  do {
    Serial.print(".") ;
    digitalWrite(LED_BUILTIN, HIGH);   //-turn the LED on
    delay(500) ;    
    digitalWrite(LED_BUILTIN, LOW);    //-turn the LED off
    delay(500);         
  }while(BBMobileSend(&bbmSerial, "<hello")) ;
  Serial.println("FOUND") ;  

  //-set BBMobile modules name
  //----------------------------------  
  Serial.print("Setting mobile name - ") ;  
  if(BBMobileSend(&bbmSerial, "<name,ARDUINO V METER"))
  {
    Serial.println("err") ;
  }else Serial.println("OK") ;  

  //-set BBMobile modules PIN
  //----------------------------------    
  //Serial.print("Setting PIN - ") ;    
  //if( BBMobileSend(&bbmSerial, "<pin,123") )  //-set PIN to 123  
  Serial.print("Setting no PIN - ") ;    
  if( BBMobileSend(&bbmSerial, "<pin,0") )  //-delete PIN
  {
    Serial.println("err") ;
  }else Serial.println("OK") ;  
}

//--------------------------------------------------
// main code here, to run repeatedly
//--------------------------------------------------
void loop()
{
  int i1, i2, i3 ;
  float v1, v2, v3 ;  
  
  //-wait for Bluetooth LE connection
  //----------------------------------    
  Serial.print("Waiting for App connection...") ;  
  do {
    digitalWrite(LED_BUILTIN, LOW);    // turn the LED off by making the voltage LOW
    delay(900) ;    
    digitalWrite(LED_BUILTIN, HIGH);   // turn the LED on (HIGH is the voltage level)
    delay(100);             
    BBMobileGetMessage(&bbmSerial) ;
  }while(BBMobileIsConnected() == 0) ;
  Serial.println("OK") ;  

  //-send JSON to build mobile interface 
  //------------------------------------
  Serial.print("Sending JSON...") ;  
  if(BBMobileSendJson(&bbmSerial, AppJson))
  {
    Serial.println("err") ;
  }else Serial.println("OK\nPlaying with interface...") ;  
  
  #ifdef DEMO
    i1 =i2 =i3 =0 ;
  #endif  //-def DEMO      
  
  //-play with interface
  //------------------------------------
  while(BBMobileIsConnected())
  {
    delay(1000) ;
    
    #ifdef DEMO
      i1++ ;
      if(i1 > 1023) i1 = 0 ;
      i2 -=11 ;
      if(i2 < 0) i2 = 1023 ;          
      i3 +=9 ;
      if(i3 > 1023) i3 = 0 ;              
    #else
      // read the input on analog pin 0:
      i1 = analogRead(A0) ;
      // read the input on analog pin 1:
      i2 = analogRead(A1) ;    
      // read the input on analog pin 2:
      i3 = analogRead(A2) ;        
    #endif  //-def DEMO    
  
    // Convert the analog reading (which goes from 0 - 1023) to a voltage (0 - 5V):
    v1 = i1 * (5.0 / 1023.0) ;
    v2 = i2 * (5.0 / 1023.0) ;
    v3 = i3 * (5.0 / 1023.0) ;
    
    //-get messages from mobile interface
    //------------------------------------
    while(BBMobileGetMessage(&bbmSerial) > 0)
    {
      //Serial.println("msg: " + bbm_buf) ;
    } ;

    //-send data to mobile interface
    //------------------------------------    
    // send v1 voltage to mobile interface
    BBMobileSend(&bbmSerial, "$set,t1:te=\"CH_1  " + String(v1, 3) + " V\"") ;        
    // send v2 voltage to mobile interface    
    BBMobileSend(&bbmSerial, "$set,t2:te=\"CH_2  " + String(v2, 3) + " V\"") ;        
    // send v3 voltage to mobile interface    
    BBMobileSend(&bbmSerial, "$set,t3:te=\"CH_3  " + String(v3, 3) + " V\"") ;        

  } ;    

  //-Here Bluetooth LE is disconnected
  //----------------------------------    
  Serial.println("BLE disconnected") ;
}
